{ config, lib, ... }:
let
  backupFolder = "~/desktop/browser-images";
  homeFolder = "/browsers";
in
{
  config = lib.mkIf config.components.gui.enable {

    # overwrite use zram on small RAM systems
    fileSystems."/share" = lib.mkDefault {
      device = "tmpfs";
      fsType = "tmpfs";
    };

    # overwrite use zram on small RAM systems
    fileSystems."${homeFolder}" = lib.mkDefault {
      device = "tmpfs";
      fsType = "tmpfs";
    };

    services.browser = {
      enable = lib.mkDefault true;
      configList = {
        development = {
          home = "${homeFolder}/development-browser";
          homeBackup = "${backupFolder}/development-browser";
          gpu = false;
        };
        google = {
          home = "${homeFolder}/google-browser";
          homeBackup = "${backupFolder}/google-browser";
          gpu = false;
        };
        finance = {
          home = "${homeFolder}/finance-browser";
          homeBackup = "${backupFolder}/finance-browser";
          gpu = false;
        };
        facebook = {
          home = "${homeFolder}/facebook-browser";
          homeBackup = "${backupFolder}/facebook-browser";
          gpu = false;
        };
        shopping = {
          home = "${homeFolder}/shopping-browser";
          homeBackup = "${backupFolder}/shopping-browser";
          gpu = false;
        };
        jobrad = {
          browserType = "chrome";
          home = "${homeFolder}/jobrad-chrome";
          homeBackup = "${backupFolder}/jobrad-chrome";
          gpu = false;
        };
        firefox-tmp = {
          browserType = "firefox";
          home = "${homeFolder}/firefox-tmp";
          homeBackup = "${backupFolder}/firefox-tmp-browser";
          gpu = false;
        };
        chromium-tmp = {
          browserType = "chrome";
          home = "${homeFolder}/chromium-tmp";
          homeBackup = "${backupFolder}/chrome-tmp-browser";
          gpu = false;
        };
        google-tmp = {
          browserType = "google";
          home = "${homeFolder}/google-tmp";
          homeBackup = "${backupFolder}google-tmp-browser";
          gpu = false;
        };

      };
    };
  };
}
