{ config, pkgs, lib, ... }:
{

  imports = [

    ../../components
    ../../modules

    ./hardware-configuration

    ./syncthing.nix
    ./tinc.nix

    ./qemu.nix
    ./wireguard.nix

    ./37c3.nix

  ];


  sops.secrets.pushover_user_key = { };
  sops.secrets.pushover_api_key = { };

  components.gui.enable = true;
  components.mainUser.enable = true;
  components.media.enable = true;
  components.media.tts-client.enable = false;
  components.network.enable = true;
  components.network.wifi.enable = true;
  components.terminal.enable = true;

  home-manager.users.mainUser.home.sessionPath = [ "$HOME/.timewarrior/scripts" ];

  sops.secrets.yubikey_u2fAuthFile = { };

  components.gui.taskwarrior.config = {
    general = {
      targets = [ "terranix" "my_github" ];
      log_level = "INFO";
      static_fields = [ "priority" ];
      merge_annotations = false;
    };
    terranix = {
      service = "github";
      login = "mrVanDalo";
      token = "@oracle:eval:${pkgs.pass}/bin/pass development/github/mrVanDalo/bugwarriorAccessToken";
      username = "mrVanDalo";
      default_priority = "";
      description_template = "{{githubtitle}} {{githuburl}}";
      add_tags = "github";
      project_template = "terranix";
      involved_issues = true;
      query = "org:terranix is:open";
      include_user_issues = false;
      include_user_repos = false;
    };
    my_github = {
      service = "github";
      login = "mrVanDalo";
      token = "@oracle:eval:${pkgs.pass}/bin/pass development/github/mrVanDalo/bugwarriorAccessToken";
      username = "mrVanDalo";
      description_template = "{{githubtitle}} {{githuburl}}";
      add_tags = "github";
      include_user_issues = true;
      include_user_repos = true;
      exclude_repos = [ "azubi" "csv-to-qif" "stepp0r" ];
    };
    # todo : add github issues
  };


  users.users.mainUser.extraGroups = [ "pipewire" ];

  services.nginx.enable = true;

  networking.hostName = "cherry";

  # make sure battery is charged in a way to live for a long time
  services.power-profiles-daemon.enable = false;
  services.tlp = {
    enable = true;
    settings = {
      CPU_BOOST_ON_BAT = 0;
      CPU_SCALING_GOVERNOR_ON_BATTERY = "powersave";
      START_CHARGE_THRESH_BAT0 = 30;
      STOP_CHARGE_THRESH_BAT0 = 85;
      RUNTIME_PM_ON_BAT = "auto";
    };
  };

  security.wrappers = {
    pmount = {
      source = "${pkgs.pmount}/bin/pmount";
      setuid = true;
      owner = "root";
      group = "root";
    };
    pumount = {
      source = "${pkgs.pmount}/bin/pumount";
      setuid = true;
      owner = "root";
      group = "root";
    };
  };

  services.printing.enable = true;

  virtualisation = {
    docker.enable = true;
    podman.enable = true;
    virtualbox = {
      host.enable = false;
      guest.x11 = false;
      guest.enable = false;
    };
  };

  # for congress and streaming
  hardware.opengl = {
    enable = true;
  };

  system.stateVersion = "23.11";

}
